<?php
/**
 * Plugin Name
 *
 * @package           PluginPackage
 * @author            Danny Boy Gomez
 * @copyright         2021 Danny Boy Gomez
 * @license           GPL-2.0-or-later
 *
 * @wordpress-plugin
 * Plugin Name:       Easy WebP
 * Plugin URI:        https://webdevphil.com/product/easy-webp
 * Description:       Boost your WordPress website Speed for FREE by up to 70% to 95% by Converting jpg, jpeg png and gif images to webp. It can covert uploaded images and theme images.
 * Version:           1.0.2
 * Requires at least: 4.0
 * Requires PHP:      5.6
 * Author:            Danny Boy Gomez
 * Author URI:        https://webdevphil.com/
 * Text Domain:       easy-webp
 * License:           GPL v2 or later
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */


if (!defined('ABSPATH')) exit; // Exit if accessed directly

global $ewp_db_version;
$ewp_db_version = '1.0';

/**
 * Activate the plugin.
 */

function ewp_activate() {

    global $wpdb;
    global $ewp_db_version;
    $table_name = $wpdb->prefix . 'easywebp';
    $table_name_converted = $wpdb->prefix . 'easywebp_converted';
    $table_name_customer_subscription = $wpdb->prefix . 'easywebp_customer_subscription';
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id mediumint(3) NOT NULL AUTO_INCREMENT,
        image_per_page mediumint(4) NOT NULL,
        quality mediumint(3) NOT NULL,
        image_type varchar(55) DEFAULT '' NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    $sql2 = "CREATE TABLE $table_name_converted (
        id int(11) NOT NULL AUTO_INCREMENT,
        original_image varchar(255) DEFAULT '' NOT NULL,
        original_size int(11) NOT NULL,
        webp_image varchar(255) DEFAULT '' NOT NULL,
        webp_size int(11) NOT NULL,
        reduce_size int(11) NOT NULL,
        date_converted datetime DEFAULT CURRENT_TIMESTAMP,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    $sql3 = "CREATE TABLE $table_name_customer_subscription (
        id int(11) NOT NULL AUTO_INCREMENT,
        email varchar(255) DEFAULT '' NOT NULL,
        site_url varchar(255) NOT NULL,
        unique_key varchar(50) DEFAULT '' NOT NULL,
        subscribe varchar(10) NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $sql );
    dbDelta( $sql2 );
    dbDelta( $sql3 );
    add_option( 'ewp_db_version', $ewp_db_version );
}


function ewp_install_data() {
    global $wpdb;
    $image_per_page = '30';
    $quality = '10';
    $image_type = 'full_image';
    $table_name = $wpdb->prefix . 'easywebp';
    $wpdb->insert( 
        $table_name, 
        array( 
            'image_per_page' => $image_per_page, 
            'quality' => $quality,
            'image_type' => $image_type, 
        ) 
    );
}

register_activation_hook( __FILE__, 'ewp_activate' );
register_activation_hook( __FILE__, 'ewp_install_data' );

/**
 * Deactivate the plugin.
 */

function ewp_deactivate() {
    // Clear the permalinks to remove our post type's rules from the database.
    flush_rewrite_rules();
}
register_deactivation_hook( __FILE__, 'ewp_deactivate' );


/* for updating db version */
global $wpdb;
$installed_ver = get_option( "ewp_db_version" );
if ( $installed_ver != $ewp_db_version ) {
    $table_name = $wpdb->prefix . 'easywebp';
    $sql = "CREATE TABLE $table_name (
        id mediumint(2) NOT NULL AUTO_INCREMENT,
        image_per_page mediumint(4) NOT NULL,
        quality mediumint(3) NOT NULL,
        image_type varchar(55) DEFAULT '' NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";
    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $sql );
    update_option( "ewp_db_version", $ewp_db_version );
}

function ewp_db_check() {
    global $ewp_db_version;
    if ( get_site_option( 'ewp_db_version' ) != $ewp_db_version ) {
        ewp_activate();
    }
}
add_action( 'plugins_loaded', 'ewp_db_check' );

// add top level menu
add_action( 'admin_menu', 'ewp_options_page' );
function ewp_options_page() {
    $hookname = add_menu_page(
        'Easy WebP - Easily Convert Your Images to WebP',
        'Easy WebP',
        'manage_options',
        plugin_dir_path(__FILE__) . 'admin/uploaded-images-easywebp.php',
        null,
        plugin_dir_url(__FILE__) . 'assets/icon_easywebp.png',
        20
    );
// add sub level menu
    $hookname = add_submenu_page(
        plugin_dir_path(__FILE__) . 'admin/uploaded-images-easywebp.php', // parent slug
        'Easy WebP - Reports',
        'Reports',
        'manage_options',
        plugin_dir_path(__FILE__) . 'admin/reports-easywebp.php',
        null,
        1
    );
    add_action( 'load-' . $hookname, 'ewp_options_page_submit' );
}



function ewp_scripts() {
    
    if ( sanitize_text_field( isset( $_GET["page"] ) == "easy-webp/admin/uploaded-images-easywebp.php" )  ||  sanitize_text_field( isset( $_GET["page"] ) == "easy-webp/admin/reports-easywebp.php" )  ) { // for wordpress front-end compatibility reason and other plugins

    $plugin_url = plugin_dir_url( __FILE__ );
    $wp_include_dir = preg_replace('/wp-content$/', 'wp-includes', content_url());

    wp_register_script( 'jquery-original', $wp_include_dir . '/js/jquery/jquery.js' , array(), '3.5.1', true );
    wp_enqueue_script( 'ajax-script', $plugin_url . 'admin/js/easy-webp-jquery.js', array(), '1.0.0', true );

    wp_enqueue_style( 'style-easy-webp',  $plugin_url . "admin/css/admin.css");
    
    wp_enqueue_script( 'script-easy-webp', $plugin_url . 'admin/js/admin.js', array(), '1.0.0', true );
    wp_enqueue_script( 'script-notification', $plugin_url . 'admin/js/sweetalert2@10.js', array(), '1.0.0', true );

    wp_enqueue_script( 'script-chart', $plugin_url . 'admin/js/chart.min.js', array(), '2.9.4', true );

    wp_enqueue_script( 'script-util', $plugin_url . 'admin/js/utils.js', array(), '1.0.0', true );

    wp_enqueue_script( 'my-script-chart', $plugin_url . 'admin/js/mycharts.js', array(), '1.0.0', true );

    wp_localize_script(
      'ajax-script',
      'my_ajax_obj',
      array(
         'ajax_url' => admin_url( 'admin-ajax.php' )
        )
    );

    }

    
    
    
}
add_action( 'admin_print_scripts', 'ewp_scripts' );


function ewp_front_scripts() {

    $plugin_url = plugin_dir_url( __FILE__ );

    wp_enqueue_script( 'modernizr-webp', $plugin_url . 'admin/js/modernizr-webp.js', array(), '3.6.0', false );

    global $wpdb;
    $table_name_converted = $wpdb->prefix . 'easywebp_converted';
    $results = $wpdb->get_results( "SELECT * FROM $table_name_converted"); 
        if ( !empty( $results ) ) {
        static $ewp_global_i;
         foreach ( $results as $row ){
            $ewp_global_i++;  
            $original_image = $row->original_image;
            $webp_image = $row->webp_image;

            $all_image = $original_image;

            // function ewp_shortcode($ewp_global_i) {

            // global $ewp_global_i;

            // $images = $original_image. " ". $webp_image;

            // return '<div class="caption">"'.$original_image.'"</div>';

            // }

            // add_shortcode( 'caption_'.$images, 'ewp_shortcode');
        


        }

    }


    

}
add_action( 'wp_enqueue_scripts', 'ewp_front_scripts' );

/* for ajax form */
add_action( 'wp_ajax_convert_single_webp', 'ewp_single_convert' );
function ewp_single_convert() {

    if ( isset( $_POST["image_link"] ) ) {
        if ( !session_id() ) {
            session_start();

        }
        if ( $_SESSION["customer_is_subscribe"] == "true" ) {
           // do nothing if sub
        } else {
            if( $_SESSION["check_used_converted_size"] <= "30") {
                // do nothing if not exceeed
            } else {
                echo esc_html( "exceed_subscription" );
                wp_die(); // all ajax handlers should die when finished
            }
        }     
        $file_to_convert = sanitize_text_field( $_POST["image_link"] ); 
        $image_quality = sanitize_text_field( $_POST["image_quality"] );
        function ewp_convert_webp( $source, $destination, $quality = '$image_quality = $image_quality' ) {
            $extension = pathinfo( $source, PATHINFO_EXTENSION );
            if ( $extension == 'jpeg' || $extension == 'jpg' ) {
                $image = imagecreatefromjpeg( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            elseif ( $extension == 'gif' )  {
                $image = imagecreatefromgif( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            elseif ( $extension == 'png' )   {
                $image = imagecreatefrompng( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            return imagewebp( $image, $destination, $quality );  
        }

        /* for creating and saving of link */
        $upload_dir   = wp_upload_dir();
        $base_upload_dir = $upload_dir['basedir'];
        $easy_webp_upload_dir = $base_upload_dir . '/easy-webp-upload/';
        $get_url = parse_url( $file_to_convert, PHP_URL_PATH );
        $get_url_exact_link = explode( '/', ( parse_url ( $file_to_convert, PHP_URL_PATH ) ) );
        $url_array_link = array_slice( $get_url_exact_link , -3, 2, true); // get the current date of link
        sanitize_text_field( $_SESSION["save_link"] = "" );
        foreach ( $url_array_link as $key => $val ) {
           sanitize_text_field( $_SESSION["save_link"] .=$val."/" );
        }
        $save_link = sanitize_text_field( $_SESSION["save_link"] );
        wp_mkdir_p( $base_upload_dir . '/easy-webp-upload/' . $save_link ); // create directory
        $save_dir = $base_upload_dir . '/easy-webp-upload/' . $save_link; // set directory to save file

        $base_upload_url = $upload_dir['baseurl'];
        $path_parts = pathinfo( $file_to_convert );
        $filename = $path_parts['filename']; // to get the filename
        $destination = $save_dir . $filename . ".webp";
        $converted_webp_url_link = $base_upload_url . '/easy-webp-upload/' . $save_link . $filename. ".webp";
        /* end of creating and saving of link and file */

        function ewp_get_single( $image_link ) {
            return @file_get_contents( $image_link, false, NULL, 0 , 1 );
        }
        if ( ewp_get_single ( $file_to_convert ) ) { // check if url file 
        $convert = ewp_convert_webp( $file_to_convert, $destination, $quality = $image_quality );

        global $wpdb;
        $original_image = $file_to_convert; 
        $webp_image = $converted_webp_url_link;
        $original_name = $path_parts['basename'];
        $original_local_file = $base_upload_dir . "/" . $save_link . $original_name; //get the original local file link in order filesize function will work
        $webp_local_file = $easy_webp_upload_dir . "/" . $save_link . $filename . ".webp"; //get the original local file link in order filesize function will work
        $original_size = filesize( $original_local_file ) / 1024;
        $webp_size = filesize( $webp_local_file ) / 1024;
        $to_convert = $original_size - $webp_size;
        $reduce_size = ( $to_convert / $original_size ) * 100;
        $table_name_converted = $wpdb->prefix . 'easywebp_converted';
        
        $wpdb->insert( 
            $table_name_converted, 
            array( 
                'original_image' => $original_image, 
                'original_size' => $original_size,
                'webp_image' => $webp_image,
                'webp_size' => $webp_size,
                'reduce_size' => $reduce_size,
            ) 
        );

        if ( $convert ) {
            echo esc_html( "Successfully Converted Image." );

        }else {
            echo esc_html( "Failed Converting Image.");

        }

        }else {
            echo esc_html( "Image file cannot be found.");
        }
    }
wp_die(); // all ajax handlers should die when finished
}


add_action( 'wp_ajax_convert_selected_webp', 'ewp_selected_convert' );
function ewp_selected_convert() {
if ( !session_id() ) {
            session_start();

        }
if ( $_SESSION["customer_is_subscribe"] == "true" ) {
   // do nothing if sub
} else {
    if( $_SESSION["check_used_converted_size"] <= "30") {
        // do nothing if not exceeed
    } else {
        echo esc_html( "exceed_subscription" );
        wp_die(); // all ajax handlers should die when finished
    }
}

        
if ( $_POST["selected_image_links"] == null || $_POST["selected_image_links"] == ""  ) {
    echo esc_html( "please_select_an_image" );
    wp_die(); // all ajax handlers should die when finished
}

function ewp_sanitize($array_or_string) {
    if( is_string($array_or_string) ){
        $array_or_string = sanitize_text_field($array_or_string);
    }elseif( is_array($array_or_string) ){
        foreach ( $array_or_string as $key => &$value ) {
            if ( is_array( $value ) ) {
                $value = ewp_sanitize($value);
            }
            else {
                $value = sanitize_text_field( $value );
            }
        }
    }

    return $array_or_string;
}
    
if ( isset( $_POST["selected_image_links"] ) ) {


        $convert_selected = ewp_sanitize( $_POST["selected_image_links"] ); 
        $max = count( $convert_selected ) - 1; // -1 because array start from 0
        $image_quality = ewp_sanitize( $_POST["image_quality"] );
        function ewp_convert_webp( $source, $destination, $quality = '$image_quality' ) {
            $extension = pathinfo( $source, PATHINFO_EXTENSION );
            if ( $extension == 'jpeg' || $extension == 'jpg' ) {
                $image = imagecreatefromjpeg( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            } elseif ( $extension == 'gif' )  {
                $image = imagecreatefromgif( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            } elseif ( $extension == 'png' )   {
                $image = imagecreatefrompng( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            if( $image == "" ) {
            }else {
                    return imagewebp( $image, $destination, $quality );
            }         
            
        }

        function ewp_get_content ( $image_file ) {
                return @file_get_contents( $image_file, false, NULL, 0 , 1 );
        }


        for ( $i = 0; $i <= $max; $i++) {

            $image_file  = $convert_selected[$i];
            /* for creating and saving of link */
            $upload_dir   = wp_upload_dir();
            $base_upload_dir = $upload_dir['basedir'];
            $easy_webp_upload_dir = $base_upload_dir . '/easy-webp-upload/';
            $get_url = parse_url( $image_file, PHP_URL_PATH );
            $get_url_exact_link = explode( '/', ( parse_url( $image_file, PHP_URL_PATH ) ) );
            $url_array_link = array_slice( $get_url_exact_link, -3, 2, true ); // get the current date of link
            sanitize_text_field( $_SESSION["save_link"] = "" );

            foreach ( $url_array_link as $key => $val ) {
               sanitize_text_field( $_SESSION["save_link"] .=$val."/" );
            }

            $save_link = sanitize_text_field( $_SESSION["save_link"] );
            wp_mkdir_p( $base_upload_dir . '/easy-webp-upload/' . $save_link ); // create directory
            $save_dir = $base_upload_dir . '/easy-webp-upload/' . $save_link; // set directory to save file

            $base_upload_url = $upload_dir['baseurl'];

            $path_parts = pathinfo($image_file);
            $filename = $path_parts['filename']; // to get the filename
            $destination = $save_dir.$filename.".webp";
            $converted_webp_url_link = $base_upload_url . '/easy-webp-upload/' . $save_link . $filename. ".webp";


            /* end of creating and saving of link and file */
            static $success_converted;
            static $failed_converted;

            if ( ewp_get_content ( $image_file ) ) { // check if url file exist
                $convert = ewp_convert_webp( $image_file, $destination, $quality = $image_quality );

                global $wpdb;
                $original_image = $image_file; 
                $webp_image = $converted_webp_url_link;
                $original_name = $path_parts['basename'];
                $original_local_file = $base_upload_dir . "/" . $save_link . $original_name; //get the original local file link in order filesize function will work
                $webp_local_file = $easy_webp_upload_dir . "/" . $save_link . $filename . ".webp"; //get the original local file link in order filesize function will work
                $original_size = filesize( $original_local_file ) / 1024;
                $webp_size = filesize( $webp_local_file ) / 1024;
                $to_convert = $original_size - $webp_size;
                $reduce_size = ( $to_convert / $original_size ) * 100;
                $table_name_converted = $wpdb->prefix . 'easywebp_converted';
                
                $wpdb->insert( 
                    $table_name_converted, 
                    array( 
                        'original_image' => $original_image, 
                        'original_size' => $original_size,
                        'webp_image' => $webp_image,
                        'webp_size' => $webp_size,
                        'reduce_size' => $reduce_size,
                    ) 
                );

                if ( $convert ) {
                $success_converted++;
                sanitize_text_field( $_SESSION["count_successfully_converted"] =  $success_converted );
                }
            } else {
                $failed_converted++;
                sanitize_text_field( $_SESSION["count_failed_converted"] =  $failed_converted ); 
            }
            if ( $_SESSION["count_successfully_converted"] != 0 ) {
                $success = "Succesfully Converted Images : " . sanitize_text_field( $_SESSION["count_successfully_converted"] );
            } else {
                $success = "";
            }
            if ( $_SESSION["count_failed_converted"] != 0 ) {
                $failed = "\nFailed to Convert : " . sanitize_text_field( $_SESSION["count_failed_converted"] );
            } else {
                $failed = "";
            }
        }
        echo esc_html( $success.$failed );
        wp_die(); // all ajax handlers should die when finished
    }
}

add_action( 'wp_ajax_change_webp_quality', 'ewp_change_quality' );
function ewp_change_quality() {
if( isset( $_POST["image_quality"]) ) {
    global $wpdb;
    $image_quality = sanitize_text_field( $_POST["image_quality"] );
    $table_name = $wpdb->prefix . 'easywebp';
    $results = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY id DESC LIMIT 1"); 
    if ( !empty( $results ) ) {

             foreach( $results as $row ){  

                $id = $row->id;
                $_SESSION["id"] = $id;

        }
    }
    $id = sanitize_text_field( $_SESSION["id"] );
    $where = [ 'id' => $id ]; 
    $sql = $wpdb->update($table_name , array('quality' => $image_quality ), $where);
    if ( $sql == TRUE) {
        echo esc_html( "Succesfully Updated Image Quality." );
        wp_die(); // all ajax handlers should die when finished

    } else {
        echo esc_html( "Error Updating Image Quality." );
        wp_die(); // all ajax handlers should die when finished
    }           
}

}

add_action( 'wp_ajax_change_image_type', 'ewp_change_type' );
function ewp_change_type() {
if( isset( $_POST["image_type"]) ) {
    global $wpdb;
    $image_type = sanitize_text_field( $_POST["image_type"] );
    $table_name = $wpdb->prefix . 'easywebp';
    $results = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY id DESC LIMIT 1"); 
    if( !empty( $results ) ) {

             foreach ( $results as $row ){ 

                $id = $row->id;
                $_SESSION["id"] = $id;

        }
    }
    $id = sanitize_text_field( $_SESSION["id"] );
    $where = [ 'id' => $id ]; 
    $sql = $wpdb->update($table_name , array('image_type' => $image_type ), $where);
    if( $sql == TRUE) {
        echo esc_html( "Succesfully Updated Image Type to Convert." );
        wp_die(); // all ajax handlers should die when finished
    }else {
        echo esc_html( "Error Updating Image Image Type to Convert." );
        wp_die(); // all ajax handlers should die when finished
    }           
    
}

}


add_action( 'wp_ajax_change_image_per_page', 'ewp_per_page' );
function ewp_per_page() {
if( isset( $_POST["image_per_page"]) ) {
    global $wpdb;
    $image_per_page = sanitize_text_field ( $_POST["image_per_page"] );
    $table_name = $wpdb->prefix . 'easywebp';
    $results = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY id DESC LIMIT 1"); 
    if( !empty( $results ) ) {
             foreach( $results as $row ){   

                $id = $row->id;
                $_SESSION["id"] = $id;

        }
    }
    $id = sanitize_text_field( $_SESSION["id"] );
    $where = [ 'id' => $id ]; 
    $sql = $wpdb->update($table_name , array('image_per_page' => $image_per_page ), $where);
    if( $sql == TRUE) {
        echo esc_html( "Succesfully Updated Image Per Page." );
        wp_die(); // all ajax handlers should die when finished
    }else {
        echo esc_html( "Error Updating Image Image Per Page." );
        wp_die(); // all ajax handlers should die when finished
    }           
}

}

add_action( 'wp_ajax_subscribe_to_easywebp', 'ewp_subscribe' );
function ewp_subscribe() {
if( isset( $_POST["email"]) && isset( $_POST["key"]) ) {
    $email = sanitize_email( $_POST["email"] );
    $site_original_url = site_url();
    //$site_original_url = "blog.danweb.net";
    $unique_key =  sanitize_text_field ( $_POST["key"] );
    $subscribe = "true";
    /* $url escaping and validation */
    $input = trim($site_original_url, '/');
    // If scheme not included, prepend it
    if (!preg_match('#^http(s)?://#', $input)) {
        $input = 'http://' . $input;
    }
    $urlParts = parse_url($input);
    $site_url_raw = preg_replace('/^www\./', '', $urlParts['host']); // remove www
    $get_url_with_sub_domain = explode('.', $site_url_raw);
    $site_url_array = array_slice($get_url_with_sub_domain, -2, 2);

    sanitize_text_field( $_SESSION["get-url"] = "" );

    foreach ( $site_url_array as $key => $val ) {
    sanitize_text_field( $_SESSION["get-url"] .=$val."." );
    }
    $refine_url = sanitize_text_field( $_SESSION["get-url"] );
    /* end of $url escaping and validation */
    $site_url = substr($refine_url, 0, -1);
    //echo $site_url;
    global $wpdb2;
    global $wpdb;
    //user, pass, database, host
    $wpdb2 = new wpdb( 'u303245712_ewp_username', 'ewp_db_Pass1', 'u303245712_ewp_db', '212.1.208.201');
    $_SESSION["customer-email"] = "";
    $table_name = 'wp_easywebp_subscription';
    $results = $wpdb2->get_results( "SELECT * FROM $table_name WHERE email='$email' AND site_url='$site_url' AND unique_key='$unique_key' AND subscribe='$subscribe' LIMIT 1"); 
    if( !empty( $results ) ) {

        foreach ( $results as $row ){   

            sanitize_text_field( $_SESSION["id"] = $row->id );
            sanitize_text_field( $customer_email = $row->email );
            sanitize_text_field( $_SESSION["customer-email"] = $customer_email );

        }
        $table_name_customer_subscription = $wpdb->prefix . 'easywebp_customer_subscription';
        $sql = $wpdb->insert( 
                    $table_name_customer_subscription, 
                    array( 
                        'email' => $email, 
                        'site_url' => $site_url,
                        'unique_key' => $unique_key,
                        'subscribe' => $subscribe,
                    ) 
                );

        if( $sql == TRUE ) {
            echo esc_html( "Succesfully Added Subscription." );
            wp_die(); // all ajax handlers should die when finished
        }
    } else {
        echo 'incorect_info';
        wp_die(); // all ajax handlers should die when finished
    }    
    
}else {
    echo "nothing";
}

}


add_action( 'wp_ajax_session_uploaded_images', 'ewp_session_uploaded' );
function ewp_session_uploaded() {
if( isset( $_POST["session_uploaded_images"] ) ) {
    if ( !session_id() ) {
            session_start();

        }
    sanitize_text_field( $_SESSION["session_theme_images"] = "false" ); // remove session theme images
    sanitize_text_field( $_SESSION["session_uploaded_images"] = "true" );
    echo esc_html( "Successfully Changed Uploaded Images to Convert." );
    wp_die(); // all ajax handlers should die when finished

}

}

add_action( 'wp_ajax_session_theme_images', 'ewp_session_theme' );
function ewp_session_theme() {
if( isset( $_POST["session_theme_images"] ) ) {
    if ( !session_id() ) {
            session_start();

        }
    sanitize_text_field( $_SESSION["session_uploaded_images"] = "false" );; // remove session uploaded images
    sanitize_text_field( $_SESSION["session_theme_images"] = "true" );
    echo esc_html( "Successfully Changed Theme Images to Convert." );
    wp_die(); // all ajax handlers should die when finished

}

}

add_action( 'wp_ajax_change_selected_theme', 'ewp_selected_theme' );
function ewp_selected_theme() {
if( isset( $_POST["selected_theme"] ) ) {
    if ( !session_id() ) {
            session_start();

        }
    sanitize_text_field( $_SESSION["selected_theme"] = "" ); // remove session theme images
    sanitize_text_field( $_SESSION["selected_theme"]  = $_POST["selected_theme"]  );
    echo esc_html( "Successfully Changed Theme to Convert." );
    wp_die(); // all ajax handlers should die when finished

}

}

/* for ajax form */
add_action( 'wp_ajax_convert_single_theme', 'ewp_single_theme' );
function ewp_single_theme() {

    if ( isset( $_POST["online_link"] ) ) {

        if ( !session_id() ) {
            session_start();

        }
        if ( $_SESSION["customer_is_subscribe"] == "true" ) {
           // do nothing if sub
        } else {
            if( $_SESSION["check_used_converted_size"] <= "30") {
                // do nothing if not exceeed
            } else {
                echo esc_html( "exceed_subscription" );
                wp_die(); // all ajax handlers should die when finished
            }
        }     
        $file_to_convert = sanitize_text_field( $_POST["online_link"] ); // online link is used for coverting
        $local_link = sanitize_text_field( $_POST["local_link"] ); // local link is used for saving
        $image_quality = sanitize_text_field( $_POST["image_quality"] );

        function ewp_theme_single( $source, $destination, $quality = '$image_quality = $image_quality' ) {
            $extension = pathinfo( $source, PATHINFO_EXTENSION );
            if ( $extension == 'jpeg' || $extension == 'jpg' ) {
                $image = imagecreatefromjpeg( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            elseif ( $extension == 'gif' )  {
                $image = imagecreatefromgif( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            elseif ( $extension == 'png' )   {
                $image = imagecreatefrompng( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            return imagewebp( $image, $destination, $quality );  
        }

        $path_parts = pathinfo( $local_link );
        $filename = $path_parts['filename']; 
        $dir_theme_image = $path_parts['dirname'];
        //$local_webp_version = $dir_theme_image . "/" . $filename . ".webp"; 

        $path_parts_online = pathinfo( $file_to_convert );
        $filename_online = $path_parts_online['filename']; 
        $dir_theme_online_image = $path_parts_online['dirname'];
        $online_webp_version = $dir_theme_online_image . "/" . $filename_online . ".webp"; // for online view like http or locahost

        $destination = $dir_theme_image . "/" . $filename . ".webp"; // forward saving e.g c:\xampp\htdocs for local

        $converted_webp_url_link = $online_webp_version;
        /* end of creating and saving of link and file */

        function ewp_get_theme( $image_link ) {
            return @file_get_contents( $image_link, false, NULL, 0 , 1 );
        }
        if ( ewp_get_theme ( $file_to_convert ) ) { // check if url file 
        $convert = ewp_theme_single( $file_to_convert, $destination, $quality = $image_quality );

        global $wpdb;
        $original_image = $file_to_convert; 
        $webp_image = $converted_webp_url_link;
        $original_name = $path_parts['basename'];
        $original_local_file = $dir_theme_image . "/" . $original_name; //get the original local file link in order filesize function will work
        $webp_local_file = $dir_theme_image . "/" . $filename . ".webp"; //get the original local file link in order filesize function will work
        $original_size = filesize( $original_local_file ) / 1024;
        $webp_size = filesize( $webp_local_file ) / 1024;
        $to_convert = $original_size - $webp_size;
        $reduce_size = ( $to_convert / $original_size ) * 100;
        $table_name_converted = $wpdb->prefix . 'easywebp_converted';
        
        $wpdb->insert( 
            $table_name_converted, 
            array( 
                'original_image' => $original_image, 
                'original_size' => $original_size,
                'webp_image' => $webp_image,
                'webp_size' => $webp_size,
                'reduce_size' => $reduce_size,
            ) 
        );

        if ( $convert ) {
            echo esc_html( "Successfully Converted Image." );
        }else {
            echo esc_html( "Failed Converting Image.");
        }

        }else {
            echo esc_html( "Image file cannot be found.");
        }

    }
wp_die(); // all ajax handlers should die when finished
}


add_action( 'wp_ajax_convert_selected_theme', 'ewp_convert_themes' );
function ewp_convert_themes() {
if ( !session_id() ) {
            session_start();

        }
if ( $_SESSION["customer_is_subscribe"] == "true" ) {
   // do nothing if sub
} else {
    if( $_SESSION["check_used_converted_size"] <= "30") {
        // do nothing if not exceeed
    } else {
        echo esc_html( "exceed_subscription" );
        wp_die(); // all ajax handlers should die when finished
    }
}

        
if ( $_POST["selected_image_links"] == null || $_POST["selected_image_links"] == ""  ) {
    echo esc_html( "please_select_an_image" );
    wp_die(); // all ajax handlers should die when finished
}

function ewp_sanitize_theme($array_or_string) {
    if( is_string($array_or_string) ){
        $array_or_string = sanitize_text_field($array_or_string);
    }elseif( is_array($array_or_string) ){
        foreach ( $array_or_string as $key => &$value ) {
            if ( is_array( $value ) ) {
                $value = ewp_sanitize_theme($value);
            }
            else {
                $value = sanitize_text_field( $value );
            }
        }
    }

    return $array_or_string;
}
    
if ( isset( $_POST["selected_image_links"] ) ) {


        $convert_selected = ewp_sanitize_theme( $_POST["selected_image_links"] );
        $convert_local_selected = ewp_sanitize_theme( $_POST["selected_local_links"] );

        $max = count( $convert_selected ) - 1; // -1 because array start from 0

        $image_quality = ewp_sanitize_theme( $_POST["image_quality"] );

        function ewp_convert_theme( $source, $destination, $quality = '$image_quality' ) {
            $extension = pathinfo( $source, PATHINFO_EXTENSION );
            if ( $extension == 'jpeg' || $extension == 'jpg' ) {
                $image = imagecreatefromjpeg( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            } elseif ( $extension == 'gif' )  {
                $image = imagecreatefromgif( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            } elseif ( $extension == 'png' )   {
                $image = imagecreatefrompng( $source );
                imagepalettetotruecolor( $image );
                imagealphablending( $image, true );
                imagesavealpha( $image, true );
            }
            if( $image == "" ) {
            }else {
                    return imagewebp( $image, $destination, $quality );
            }         
            
        }

        function ewp_get_themes ( $image_file ) {
                return @file_get_contents( $image_file, false, NULL, 0 , 1 );
        }


        for ( $i = 0; $i <= $max; $i++) {

            $image_file  = $convert_selected[$i];

            $image_local_file  = $convert_local_selected[$i];

            $path_parts = pathinfo( $image_local_file );
            $filename = $path_parts['filename']; 
            $dir_theme_image = $path_parts['dirname'];
            //$local_webp_version = $dir_theme_image . "/" . $filename . ".webp"; 

            $path_parts_online = pathinfo( $image_file );
            $filename_online = $path_parts_online['filename']; 
            $dir_theme_online_image = $path_parts_online['dirname'];
            $online_webp_version = $dir_theme_online_image . "/" . $filename_online . ".webp"; // for online view like http or locahost

            $destination = $dir_theme_image . "/" . $filename . ".webp"; // forward saving e.g c:\xampp\htdocs for local

            $converted_webp_url_link = $online_webp_version;

            /* end of creating and saving of link and file */
            static $success_converted;
            static $failed_converted;

            if ( ewp_get_themes ( $image_file ) ) { // check if url file exist
                $convert = ewp_convert_theme( $image_file, $destination, $quality = $image_quality );

                global $wpdb;
                $original_image = $image_file; 
                $webp_image = $converted_webp_url_link;
                $original_name = $path_parts['basename'];
                $original_local_file = $dir_theme_image . "/" . $original_name; //get the original local file link in order filesize function will work
                $webp_local_file = $dir_theme_image . "/" . $filename . ".webp"; //get the original local file link in order filesize function will work
                $original_size = filesize( $original_local_file ) / 1024;
                $webp_size = filesize( $webp_local_file ) / 1024;
                $to_convert = $original_size - $webp_size;
                $reduce_size = ( $to_convert / $original_size ) * 100;
                $table_name_converted = $wpdb->prefix . 'easywebp_converted';
                
                $wpdb->insert( 
                    $table_name_converted, 
                    array( 
                        'original_image' => $original_image, 
                        'original_size' => $original_size,
                        'webp_image' => $webp_image,
                        'webp_size' => $webp_size,
                        'reduce_size' => $reduce_size,
                    ) 
                );

                if ( $convert ) {
                $success_converted++;
                $_SESSION["count_successfully_converted"] =  $success_converted;
                }
            } else {
                $failed_converted++;
                $_SESSION["count_failed_converted"] =  $failed_converted; 
            }
            if ( $_SESSION["count_successfully_converted"] != 0 ) {
                $success = "Succesfully Converted Images : " . sanitize_text_field( $_SESSION["count_successfully_converted"] );
            } else {
                $success = "";
            }
            if ( $_SESSION["count_failed_converted"] != 0 ) {
                $failed = "\nFailed to Convert : " . sanitize_text_field( $_SESSION["count_failed_converted"] );
            } else {
                $failed = "";
            }
        }
        echo esc_html( $success.$failed );
        wp_die(); // all ajax handlers should die when finished
    }
}

/* register session */
function ewp_session(){
    if( !session_id() )
    session_start();
}
add_action('init','ewp_session');